#!/usr/bin/python
# Copyright (C) 2013 Petter Reinholdtsen <pere@hungry.com>
#
# Licensed under the GNU General Public License Version 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

__author__ = "Petter Reinholdtsen <pere@hungry.com>"

import sys
import apt
import fnmatch
import os
import urllib
import errno

pkgdir = "/usr/share/isenkram"

def my_modaliases():
    modaliases = set()
    for dirpath, _, filenames in os.walk("/sys/devices"):
        if "modalias" in filenames:
            with open(dirpath + "/modalias") as sysfile:
                modaliases.add(sysfile.read().strip())
    with open ("/proc/modules") as procfile:
        line = procfile.readline()
        while line:
            modalias = "lkmodule:%s" % (line.split(" "))[0]
            modaliases.add(modalias)
            line = procfile.readline()
        procfile.close()
    return list(modaliases)

def modalias_match(alias, candidates):
#    print "modalias_match('%s', %s)" % (alias, candidates)
    for part in candidates.split(')'):
        part = part.strip(', ')
        if not part:
            continue
        module, lst = part.split('(')
        for candidate in lst.split(','):
            candidate = candidate.strip()
            bus = candidate.split(':', 1)[0]
#            print bus, alias, candidate
#            print candidate, alias
            if fnmatch.fnmatch(alias, candidate):
#                print "Match"
                return True
    return False

def pkgs_handling_apt_modaliases(modaliaslist):
    cache = apt.Cache()
    cache.open(None)
    thepkgs = {}
    for pkg in cache:
        version = pkg.candidate
        if version is None:
            version = pkg.installed
        if version is None:
            continue
        record = version.record
        if not record.has_key('Modaliases'):
            continue
        modaliases = record['Modaliases']
        for modalias in modaliaslist:
            if modalias_match(modalias, modaliases):
                thepkgs[pkg.name] = True
    return thepkgs.keys()

def check_packages_file(pkghash, fileref, modaliaslist):
    line = fileref.readline()
    while line:
        header, sep, packagename = line.strip().partition(": ")
        header, sep, modaliases = fileref.readline().strip().partition(": ")
        blank = fileref.readline()
        for modalias in modaliaslist:
#            print "Match ", packagename, " ", modalias, " ", modaliases
            if modalias_match(modalias, modaliases):
                pkghash[packagename] = True
        line = fileref.readline()
    fileref.close()

def pkgs_handling_extra_modaliases(modaliaslist):
    """
Look up package-hardware mappings from svn and local file
"""
    thepkgs = {}
    url = "http://anonscm.debian.org/gitweb/?p=collab-maint/isenkram.git;a=blob_plain;f=modaliases;hb=HEAD"
    try:
        f = urllib.urlopen(url)
        check_packages_file(thepkgs, f, modaliaslist)
    except IOError, e:
        if errno.ENOENT == e.errno: # Most likely lack network connection
            pass
    try:
        try:
            f = open("modaliases")
            check_packages_file(thepkgs, f, modaliaslist)
        except IOError as e:
            f = open(pkgdir + "/modaliases")
            check_packages_file(thepkgs, f, modaliaslist)
        f.close()

        for dirpath, _, filenames in os.walk(pkgdir + "/modaliases.d"):
            for filename in filenames:
                with open(dirpath + "/modalias") as aliasfile:
                    check_packages_file(thepkgs, aliasfile, modaliaslist)

    except: # Ignore errors
        pass
#    print thepkgs
    return thepkgs.keys()

def main():
    hwaliases = my_modaliases()

    if 1 < len(sys.argv):
        hwaliases = sys.argv[1:]

    print "Locating packages supporting this hardware (extra, svn/local):"
    for pkg in sorted(pkgs_handling_extra_modaliases(hwaliases)):
        print "  %s" %pkg

    print "Locating packages supporting this hardware (APT):"
    for pkg in sorted(pkgs_handling_apt_modaliases(hwaliases)):
        print "  %s" %pkg

if __name__ == '__main__':
    main()
